# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevelMVNormal(Template):
    'Fits 2-level multivariate Normal response models, allowing for missing data, and a choice of uniform or Wishart priors (although only Wishart supported by WinBUGs / OpenBUGS).'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Multivariate response', 'MLwiN:point & click']
    engines = ['eStat', 'OpenBUGS', 'MLwiN_MCMC', 'MLwiN_IGLS', 'R_MCMCglmm', 'R_nimble', 'GenStat_model']

    inputs = '''
y = DataMatrix('Responses: ')
L2ID = IDVector('Level 2 ID: ')

for i in range(0, len(y)):
    context['x'+str(i+1)] = DataMatrix('Explanatory variables for response ' + y[i] + ': ', allow_cat = True)

storeresid = Boolean('Store level 2 residuals?')
mv = Boolean('Use MVNormal update for beta?: ')

lenbeta = 0
for i in range(0, len(y)):
    lenbeta += len(context['x'+str(i+1)])
    context['miss'+y[i]] = ParamVector(monitor=False)
    if storeresid:
        context['u' + str(i)] = ParamVector(parents=[L2ID], as_scalar=False)
    else:
        context['u' + str(i)] = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)

n = len(y)
if n == 1:
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
    tau_u = ParamScalar()
    sigma_u = ParamScalar(modelled = False)
else:
    omega_e = ParamMatrix(modelled = False, customstep=True)
    omega_e.size = n
    omega_u = ParamMatrix(modelled = False, customstep=True)
    omega_u.size = n
    d_e = ParamMatrix(customstep=True)
    d_e.size = n
    d_u = ParamMatrix(customstep=True)
    d_u.size = n
    priors = Text('Priors (NB Uniform not supported by WinBUGS / OpenBUGS): ', ['Uniform', 'Wishart'])
    if priors == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')
    priors2 = Text('Level 2 priors (NB Uniform not supported by WinBUGS / OpenBUGS): ', ['Uniform', 'Wishart'])
    if priors2 == 'Wishart':
        R2 = List('R matrix: ')
        v2 = Integer('Degrees of Freedom:')

if mv:
    beta = ParamVector(customstep=True)
else:
    beta = ParamVector()
beta.ncols = lenbeta

deviance = ParamScalar(modelled=False) 
'''

    model = '''
<% lenbeta = 0 %> \\
<% n = len(context['y']) %> \\
model {
    for (i in 1:length(${y[0]})) {
    % if n == 1:
            miss0[i] ~ dnorm(mu0[i, j], tau)
            mu0[i] <- ${mmult2(context['x1'], 'beta', 'i', lenbeta)} + u0[j]
<% lenbeta += len(context['x1']) %> \\
    % else:
            dummy[i] ~ dnormal${n}a( \\
        % for i in range(0,n):
miss${y[i]}[i], \\
        % endfor
        % for i in range(0,n):
mu${i}[i], \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_e[${i}], \\
        % endfor
d_e[${(((n+1)*n)/2) - 1}])
        % for i in range(0, len(y)):
            mu${i}[i] <- ${mmult2(context['x' + str(i+1)], 'beta', 'i', lenbeta)} + u${i}[${L2ID}[i]]
<% lenbeta += len(context['x' + str(i+1)]) %> 
        % endfor
        % for i in range(0,n):
        miss${y[i]}[i] ~ dflat()
        % endfor
    % endif
    }
      
    for (j in 1:length(u0)) {
        %if len(y) == 1 :
           u0[j] ~ dnorm(0, tau_u)
        % else:
        dummy_u[j] ~ dnormal${n}a( \\
        % for i in range(0,n):
u${i}[j], \\
        % endfor
        % for i in range(0,n):
0, \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_u[${i}], \\
        % endfor
d_u[${(((n+1)*n)/2) - 1}])
        dummy2_u[j] ~ ddummy(dummy_u[j])
        % for i in range(0,n):
        u${i}[j] ~ dflat()
        % endfor

    % endif

    }

    # Priors
    
    % if n == 1:
    tau ~ dgamma(0.001, 0.001)
    sigma <- 1 / sqrt(tau)
    tau_u ~ dgamma(0.001, 0.001)
    sigma_u <- 1 / sqrt(tau_u)
    % endif

    % for i in range(0, lenbeta):
    beta_${i} ~ dflat()
    % endfor   
}       
'''        

    latex = r'''
<% lenbeta = 0 %>
\begin{aligned} 
\left(
\begin{array}{l}
% for i in range(0, len(y)):
\mbox{${y[i]}}_i 
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
\mu_{${i}i}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_e \right] \\ 
% for i in range(0, len(y)):
\mu_{${i}i} & = ${mmulttex2(context['x' + str(i+1)], r'\beta', 'i', lenbeta)} + u_{${i},${L2ID}(i)} \\ 
<% lenbeta += len(context['x' + str(i+1)]) %>
% endfor

\left(
\begin{array}{l}
% for i in range(0, len(y)):
u_{${i},${L2ID}(i)}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
0
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_{u} \right] \\ 
%if str(Engine) in ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS' , 'MLwiN_MCMC', 'R_MCMCglmm', 'R_nimble' ]:
% if priors == 'Wishart':
\Omega_e & \sim \mbox{inverse Wishart}_{${len(y)}}[${v}*R_e,${v}],R_e=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${R.name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors == 'Uniform':
\Omega_e & \propto 1
% endif
\\ 
% if priors2 == 'Wishart':
\Omega_u & \sim \mbox{inverse Wishart}_{${len(y)}}[${v2}*R_u,${v2}],R_u=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${R2.name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors2 == 'Uniform':
\Omega_u & \propto 1
% endif
\\ 
% for i in range(0, lenbeta):
\beta_{${i}} & \propto 1 \\ 
% endfor
%endif
\end{aligned} 
'''

    preccode = '''
        <%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += 'double(' + name + '[' + index + ']) * ' + var + '_' + str(count)
            count += 1
        return out
%>
    {
<% n = len(y) %>\\
        // Note currently using a uniform prior for variance matrix
        SymMatrix sbe(${n});
        int vwe = length(${y[0]});
        for(int i = 0; i < length(${y[0]}); i++) {
<% lenbeta = 0 %>\\
% for i in range(0, n):
            double e${i} = double(miss${y[i]}[i]) - (${mmult2(context['x' + str(i+1)], 'beta', 'i', lenbeta)}) - double(u${i}[int(${L2ID}[i])]);
<% lenbeta += len(context['x' + str(i + 1)]) %>\\
% endfor

% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }
% if priors == 'Uniform':
        if (runstate == 0) {
% for i in range(0, n):
            sbe(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0 
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R.name[count]) 
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
        sbe(${i}, ${j}) += ${str(Rmat[i][j] * float(v))};
% endfor
% endfor
% endif
        matrix_sym_invinplace(sbe);

% if priors == 'Uniform':
        vwe -= ${n + 1};
% endif
% if priors == 'Wishart':
        vwe += ${v};
% endif

        bool fail = false;
        mat_d_e = dwishart(vwe, sbe, fail);
        mat_omega_e = matrix_sym_inverse(mat_d_e, fail);

        std::vector<double*> tmp_u;
% for i in range(0, n):
        tmp_u.push_back(u${i});
% endfor
RectMatrix mat_u(tmp_u, length(u0));

        SymMatrix sbu = mat_u.T() * mat_u;

% if priors2 == 'Uniform':
        if (runstate == 0) {
% for i in range(0, n):
            sbu(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if priors2 == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0 
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R2.name[count]) 
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
        sbu(${i}, ${j}) += ${str(Rmat[i, j] * float(v2))};
% endfor
% endfor
% endif

        matrix_sym_invinplace(sbu);
% if priors2 == 'Uniform':
        int vwu = length(u0) - ${n + 1};
% endif
% if priors2 == 'Wishart':
        int vwu = length(u0) + ${v2};
% endif
        mat_d_u = dwishart(vwu, sbu, fail);
        mat_omega_u = matrix_sym_inverse(mat_d_u, fail);


% if mv:
                // Step for beta
                std::vector<int> betaind(${lenbeta});
<% betacount = 0 %>
% for i in range(0, n):
% for j in range(0, len(context['x' +str(i+1)])):
                betaind[${betacount}] = ${i};
<% betacount += 1 %>
% endfor
% endfor
                std::vector<double*> tmp_x;
% for i in range(0, n):
% for j in range(0, len(context['x' + str(i+1)])):
                tmp_x.push_back((double*)${context['x' + str(i+1)][j]});
% endfor
% endfor
                RectMatrix mat_x(tmp_x, length(${y[0]}));

                std::vector<double*> tmp_y;
% for i in range(0, n):
                tmp_y.push_back(miss${y[i]});
% endfor
                RectMatrix mat_y(tmp_y, length(${y[0]}));

                SymMatrix db(${lenbeta});

                for (int i = 0; i < length(${y[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta}; j1++) {
                        for (int j2 = 0; j2 < j1+1; j2++) {
                            db(j1, j2) += mat_x(i, j1) * mat_d_e(betaind[j1], betaind[j2]) * mat_x(i, j2);
                        }
                    }
                }

                RectMatrix xtdy(${lenbeta}, 1);
                for (int i = 0; i < length(${y[0]}); i++) {
                    for (int j1 = 0; j1 < ${lenbeta}; j1++) {
                        for (int j2 = 0; j2 < ${n}; j2++) {
                            xtdy(j1, 0) += mat_x(i, j1) * mat_d_e(betaind[j1], j2) * (mat_y(i, j2) - mat_u(${L2ID}[i], j2));
                        }
                    }
                }

                fail = false;
                //int err = matrix_sym_invinplace(db);
                //RectMatrix betahat = db * xtdy;
                //mat_beta = dmultnormal(betahat, db, fail);

                RectMatrix betahat = matrix_solve(db, xtdy);
                mat_beta = dmultnormalinv(betahat, db, fail);

% endif

    }
         '''

    bugsdata = '''
import numpy
        
# Y datamatrix
# X datamatrix with x1 for variable 1 with x1, x2 for variable 2
        
N=dsets['datafile'].nobs
M=len(y)

consts['N'] = N
consts['M'] = M
consts['n1'] = len(list(set(dsets['datafile'].variables[L2ID]['data'])))
        
#start with R structure
Rmat=numpy.zeros((M,M))
k=0
        
for i in range(M):
    for j in range(i+1):
        Rmat[i,j]=float(R[k])
        Rmat[j,i]=float(R[k])
        k+=1

consts['R'] = Rmat


#Next do R2 structure
R2mat=numpy.zeros((M,M))
k=0
        
for i in range(M):
    for j in range(i+1):
        R2mat[i,j]=float(R2[k])
        R2mat[j,i]=float(R2[k])
        k+=1

consts['R2'] = R2mat


# L2 ID
uniq = numpy.unique(dsets['datafile'].variables[L2ID]['data'].compressed())
l2id = numpy.zeros(len(dsets['datafile'].variables[L2ID]['data']))
map = zip(range(1, len(dsets['datafile'].variables[L2ID]['data'])+1), uniq)
d = {}
for i in map:
    d[i[1]] = i[0]
for i in range(0, len(dsets['datafile'].variables[L2ID]['data'])):
    l2id[i] = d[dsets['datafile'].variables[L2ID]['data'][i]]
consts[L2ID] = l2id

# x variables
for i in range(M):
    for j in range(len(context['x'+str(i+1)])):
        data[context['x'+str(i+1)][j]] = dsets['datafile'].variables[context['x'+str(i+1)][j]]['data']
                        
# Y structure
Ymat=numpy.ma.zeros((N,M))
for i in range(M):
    Ymat[:,i] = dsets['datafile'].variables[y[i]]['data']

data['Y'] = Ymat
'''
    
    bugsmodel ='''
<% M = len(y) %>
<% lenbeta = 0 %>
        
        model {
            for (i in 1:N) {
% if M == 1:
                Y[i] ~ dnorm(mu0[i], tau)
                mu0[i] <- ${mmult2(context['x1'],'beta','i',lenbeta)}
<%lenbeta += len(context['x1']) %>\\
            }
% else: 
                Y[i,1:M] ~ dmnorm(mu[i,1:M], d_e[1:M,1:M])
% for i in range(0,len(y)):
                mu[i, ${i+1}] <- ${mmult2(context['x'+ str(i+1)],'beta','i',lenbeta)} + u[${L2ID}[i],${i+1}]
<% lenbeta += len(context['x' + str(i+1)]) %>\\
% endfor
            }
% endif
            for (i in 1:n1) {
               u[i,1:M] ~ dmnorm(zero[1:M], d_u[1:M,1:M])
            }
            # Priors
            # for fixed effects
% for i in range(0,lenbeta):
            beta_${i} ~ dflat()
% endfor
            for (i in 1:M)
            {
               zero[i] <- 0
            }
            #for random terms
% if M==1 :
            tau ~ dgamma(0.001,0.001)
% else:
            d_e[1:M, 1:M] ~ dwish(R[,], ${v})
            omega_e[1:M, 1:M] <- inverse(d_e[1:M, 1:M])
            d_u[1:M, 1:M] ~ dwish(R2[,], ${v2})
            omega_u[1:M, 1:M] <- inverse(d_u[1:M, 1:M])
% endif
        }
''' 

    mlnscript = '''
<% M=len(y) %>
% if M==1 :
ADDT "_levres"
SETV 1 "_levres"
FPAR 0 "_levres"
% for i in range(0, len(y)):
% for j in range(len(context['x'+str(i+1)])):
ADDT "${context['x'+str(i+1)][j]}"
% endfor
% endfor
% else:
% for i in range(len(y)):
MVAR 1 "${y[i]}"
% endfor
IDEN 3 "${L2ID}"
IDEN 2 "_id"
IDEN 1 "resp_indicator"

<%
import numpy
T=numpy.eye(len(y), dtype='int')
%>
% for j in range(0, len(T)):
RPAT ${' '.join([str(k) for k in T[j]])}
% for i in range(0,len(context['x'+str(j+1)])):
ADDT "${context['x'+str(j+1)][i]}"
% endfor
ADDT "_levres"
RPAT
% endfor

% for i in range(len(y)):
SETV 3 "_levres.${i + 1}"
SETV 2 "_levres.${i + 1}"
FPAR 0 "_levres.${i + 1}"
% endfor

<%
import numpy

TT = ' '.join([str(i) for i in numpy.zeros(int(beta.ncols),dtype='int')])

if priors == 'Wishart':
    TR = ' '.join(R.name)

if priors2 == 'Wishart':
    TR2 = ' '.join(R2.name)
%>
% if priors2 == 'Wishart' and priors !='Wishart':
JOIN ${TT} 1 ${TR2} ${str(v2)} 0 0 c1092
PRIOR  c1092 
% endif
% if priors == 'Wishart' and priors2 !='Wishart':
JOIN ${TT} 0 1 ${TR} ${str(v)} 0 c1092
PRIOR  c1092 
% endif
% if priors == 'Wishart' and priors2 =='Wishart':
JOIN ${TT} 1 ${TR2} ${str(v2)} 1 ${TR} ${str(v)} 0 c1092
PRIOR  c1092 
% endif
% endif
        '''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bf{Open}\text{ }\bf{Sample}\text{ } \bf{Worksheet} \text{ or }\bf{Open}\text{ } \bf{Worksheet} \text{ from the }\bf{File}\text{ menu.} \\ 
&\text{Select }\bf{datafile.dta}.\\ 
&\text{Select }\bf{Equations}\text{ from the }\bf{Model}\text{ menu.}\\ 
&\text{Click on the } \bf{Responses}\text{ button at the bottom of the window.}\\ 
% for i in range(len(y)):
&\text{Click on }\bf{${y[i]}}\text{ from the list that appears.}\\ 
% endfor
&\text{Click on the }\bf{Done}\text{ button on the }\bf{Responses} \text{ window.}\\ 
&\text{On the }\bf{Equations}\text{ window click on the response names to bring up the }\mathbf{Y}\text{ }\bf{variable}\text{ window}.\\ 
&\text{On the }\mathbf{Y}\text{ }\bf{variable}\text{ window change the number of levels to } \mathbf{3}-\mathbf{ijk}.\\ 
&\text{In the }\bf{level} \text{ }\mathbf{3(k):}\text{ list, select } \bf{${L2ID}}.\\  
&\text{In the }\bf{level} \text{ }\mathbf{2(j):}\text{ list, select } \bf{\_id}.\\  
&\text{Click on the }\bf{Done}\text{ button.}\\ 
&\text{Click the }\bf{Add}\text{ }\bf{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bf{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bf{Add}\text{ }\bf{Separate}\text{ }\bf{coefficients}\text{ button.}\\ 
% for i in range(len(y)):
&\text{On the }\bf{Equations}\text{ window click on }\bf{\_levres.${y[i]}}.\\ 
&\text{Remove the tick in the }\bf{Fixed}\text{ }\bf{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }\mathbf{k}(\bf{${L2ID}\_long})\text{ tick box.}\\ 
&\text{Add the tick in the }\mathbf{j}(\bf{\_id\_long})\text{ tick box.}\\ 
&\text{Click on the }\bf{Done}\text{ button.}\\ 
% endfor
<%
tt=list()
for i in range(0, len(y)):
    for j in range(len(context['x'+str(i+1)])):
        tt.append( context['x'+str(i+1)].name[j])

tt=list(set(tt))
%>
% for i in range(len(tt)):
&\text{Click the }\bf{Add}\text{ }\bf{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bf{${tt[i]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bf{Add}\text{ }\bf{Separate}\text{ }\bf{coefficients}\text{ button.}\\ 
% endfor
%for i in range(0, len(y)):
%for j in range(len(tt)):
%if tt[j] not in context['x'+str(i+1)].name:
&\text{On the }\bf{Equations}\text{ window click on }\bf{${tt[j]}.${y[i]}}.\\ 
&\text{Remove the tick in the }\bf{Fixed}\text{ }\bf{parameter}\text{ tick box.}\\ 
&\text{Click on the }\bf{Done}\text{ button.}\\ 
%endif
%endfor
%endfor
'''

    latexMLwiNextra = r'''
&\text{Select the }\bf{MCMC}\text{ tab from the }\bf{Estimation}\text{ }\bf{control}\text{ window.}\\ 
&\text{Select }\bf{MCMC}/\bf{Priors}\text{ from the }\bf{Model}\text{ menu.}\\ 
&\text{Select }\bf{Gamma}\text{ }\bf{prior}\text{ from the } \bf{MCMC}\text{ }\bf{Priors}\text{ window (this sets the prior for level 1 variance).}\\ 
% if priors2 == 'Wishart':
&\text{Select }\bf{Informative}\text{ }\bf{prior} \text{ from the } \bf{MCMC}\text{ }\bf{Priors}\text{ window.}\\ 
&\text{Select }\Sigma_v \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R2.name)):
%if i==(len(R2.name)-1):
\text{and }\mathbf{${R2.name[i]}}
%else:
\mathbf{${R2.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v2.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_v \text{ again.}\\ 
%endif
% if priors == 'Wishart':
&\text{Select }\Sigma_u \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R.name)):
%if i==(len(R.name)-1):
\text{and }\mathbf{${R.name[i]}}
%else:
\mathbf{${R.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_u \text{ again.}\\ 
%endif
&\text{Click on the }\bf{Done}\text{ button.}\\ 
'''
    
    rscript = '''
<%
import re
from EStat.Templating import Rmmult
M=len(y)

resp='cbind('
nparams=0
for i in range(M):
    resp+=str(y[i])
    nparams+=len(context['x'+str(i+1)])
    if i!= M-1:
        resp+=','
    else:
        resp+=')'
TT=1
for i in range(M-1,0,-1):
    if len(context['x'+str(i+1)])!=len(context['x'+str(i)]):
        TT=0
    else:
        aa=context['x'+str(i+1)].name
        aa.sort()
        bb=context['x'+str(i)].name
        bb.sort()
        for j in range(len(aa)):
            if aa[j]!=bb[j]:
                TT=0
    
%>

% if Rpackage == 'MCMCglmm':

% if TT!=0:
formula <- ${resp} ~ trait:(${Rmmult(context['x1'])}) - 1

##define the prior...use non-informative prior #fixed effect N(0,10^6), variance~IW(1,0.002)~IG(0.001000,0.001000)
#random effect Uniform: G1~IW(vw2,diag(M)); Wishart: Rmat2 
#Residual Uniform: G1~IW(vw,diag(M)); Wishart: Rmat
% if priors2.name == 'Uniform':
vw2 = ${M + 1};
% endif
% if priors2.name == 'Wishart':
vw2 = ${v2};
Rmat2 = matrix(0,${M},${M})
Rvec2=rep(0,${len(R2.name)})
%for i in range(0,len(R2)):
Rvec2[${i+1}]=as.numeric(${float(R2[i])})
%endfor
count = 0 
for (j in  0:${M-1}){
    for (k in 0:j ){
        Rmat2[j+1, k+1] = as.numeric(Rvec2[count+1])
        Rmat2[k+1, j+1] = Rmat2[j+1, k+1]
        count = count+ 1
    }
}
% endif

% if priors.name == 'Uniform':
vw = ${M + 1};
% endif
% if priors.name == 'Wishart':
vw = ${v};
Rmat = matrix(0,${M},${M})
Rvec=rep(0,${len(R)})
%for i in range(0,len(R)):
Rvec[${i+1}]=as.numeric(${float(R[i])})
%endfor
count = 0 
for (j in  0:${M-1}){
    for (k in 0:j ){
        Rmat[j+1, k+1] = as.numeric(Rvec[count+1])
        Rmat[k+1, j+1] = Rmat[j+1, k+1]
        count = count+ 1
    }
}
% endif

% if priors.name == 'Uniform' and priors2.name == 'Uniform':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=diag(${M}),nu=vw2)),R=list(V=diag(${M}),nu=vw))
% endif
% if priors.name == 'Wishart' and priors2.name == 'Uniform':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=diag(${M}),nu=vw2)),R=list(V=Rmat,nu=vw))
% endif
% if priors.name == 'Uniform' and priors2.name == 'Wishart':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=Rmat2,nu=vw2)),R=list(V=diag(${M}),nu=vw))
% endif
% if priors.name == 'Wishart' and priors2.name == 'Wishart':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(G1=list(V=Rmat2,nu=vw2)),R=list(V=Rmat,nu=vw))
% endif



myModel<-MCMCglmm(formula,random=~us(trait):${L2ID}, rcov=~us(trait):units,family=rep("gaussian",${M}),prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata,verbose=FALSE)

##summary and outputs from the model
## combined output fixed and variance into one matrix 

summary(myModel)


myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)
summary(myModelOut)

##temporary plots
<% ind=divmod(nparams,3) %>
% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor
% if (ind[1]>0):
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off()
% endif
## export chains
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")


%else:
<%print 'The model not implemented yet, choose another engine:'%>
%endif

% endif
'''

    genstatscript = '''
<%import numpy %>
UNITS [NVALUES=*]
DELETE[REDEFINE=yes] ${L2ID}
FACTOR [MODIFY=no; NVALUES=${len(datafile['datafile'][str(L2ID)])}; LEVELS=${len(numpy.unique(datafile['datafile'][str(L2ID)]))}; REFERENCE=1] ${L2ID}
READ ${L2ID}; FREPRESENTATION=ordinal
<%
tempstr =  ''
random=datafile['datafile'][str(L2ID)]
count =1
for d in random:
    if count==100:
        tempstr =tempstr +str(d)+' \\n'
        count=1
    else:
        tempstr =tempstr +str(d)+" "
    count+=1
    
tempstr =tempstr +":"
%>
${tempstr}
<%
x0=[]
for p in range(len(context['x1'])):
    if context['x1'].name[p]=='cons':
        flagcons=True
    else:
        x0.append(context['x1'].name[p])

%>

DELETE [REDEFINE=yes] %_units, %_variable, _Data,\\
%for i in range(len(x0)):
%%_${x0[i]},\\
%endfor
%%_${L2ID}
FACTOR [LEVELS=${len(random)}] %_units;!((1...${len(random)})${len(y)})
FACTOR [LEVELS=${len(y)}; LABELS=!t(\\
%for i in range(len(y)):
%if i != len(y)-1:
'${y[i]}',\\
%else:
'${y[i]}'\\
%endif
%endfor
)] %_variable;!(${len(random)}(1...${len(y)}))
APPEND [NEW=_Data] \\
%for i in range(len(y)):
%if i != len(y)-1:
${y[i]},\\
%else:
${y[i]}
%endif
%endfor
%for i in range(len(x0)):
APPEND [NEW=%_${x0[i]}] ${len(y)}(${x0[i]})
%endfor
APPEND [NEW=%_${L2ID}] ${len(y)}(${L2ID})

"Calculate Initial Values for Multivariate Linear Mixed Model"
DELETE [REDEFINE=yes] _resid, _mresid, _vinit, _count
VCOMP [FIXED=\\
%for i in range(len(x0)):
%if i != len(x0)-1:
%%_variable/(%_${x0[i]})+\\
%else:
%%_variable/(%_${x0[i]})\\
%endif
%endfor
; CONST=omit] (%_${L2ID}).%_variable; CONSTRAIN=positive
REML [PRINT=*; MAXCYCLE=20; FMETHOD=automatic; PSE=differences; MVINCLUDE=explanatory,\
yvariate; METHOD=AI] _Data; RESIDUAL = _resid
CALC _resid = MVREPLACE(_resid;0)
MATRIX [ROW=${len(y)}; COLUMN=${len(random)};VALUE=#_resid ] _mresid
SYMM [ROW=${len(y)}] _vinit
CALC _count = (_mresid.NE.C('*'))*+T(_mresid.NE.C('*'))
CALC _vinit = _mresid *+ T(MVREPLACE(_mresid;0)) / _count

"Fit Multivariate Linear Mixed Model"
VCOMP [FIXED=\\
%for i in range(len(x0)):
%if i != len(x0)-1:
%%_variable/(%_${x0[i]})+\\
%else:
%%_variable/(%_${x0[i]})\\
%endif
%endfor
; FACTORIAL=9; CONST=omit] (%_${L2ID}).%_variable + %_units.%_variable;\
 CONSTRAIN=positive
VSTRUCTURE [%_${L2ID}.%_variable] FACTOR=%_variable; MODEL=unstructured
VSTRUCTURE [%_units.%_variable] FACTOR=%_variable; MODEL=unstructured; INITIAL=!(#_vinit)
REML [PRINT=model,components,effects,waldTests; MAXCYCLE=20; FMETHOD=automatic; PSE=differences;\
 MVINCLUDE=explanatory,yvariate; METHOD=AI] _Data

SETDEVICE 'RCHECK.png'; NUMBER=Device
PRINT     Device; DECIMALS=0
VPLOT fittedvalues,normal,halfnormal,histogram
CLOSE Device; FILETYPE=graphics
ENDJOB
'''

    preparedata = '''
mydata = data['datafile']

for i in range(0, len(y)):
    context['miss'+y[i]].ncols = -1*len(mydata.variables[y[i]]['data'])
'''
