# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateMVNlev(Template):
    'Fits multivariate Normal response models with one or more higher classifications (nested or crossed), allowing for missing data, and a choice of uniform (not supported by WinBUGS / OpenBUGS) or Wishart priors.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Multivariate response', 'MLwiN:point & click' ]
    engines = ['eStat', 'OpenBUGS', 'MLwiN_MCMC', 'R_MCMCglmm', 'R_nimble']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
y = DataMatrix('Responses: ')

for i in range(0, len(y)):
    context['x'+str(i+1)] = DataMatrix('Explanatory variables for response ' +y[i] + ': ', allow_cat = True)

storeresid = Boolean('Store residuals?')

lenbeta = 0
for i in range(0, len(y)):
    lenbeta += len(context['x'+str(i+1)])
    context['miss' + y[i]] = ParamVector(monitor=False)
    for j in range(0, int(NumLevs)):
        if storeresid:
            context['u' + str(j) + '_' + str(i)] = ParamVector(parents=[context['C' + str(j + 1)]], as_scalar=False)
        else:
            context['u' + str(j) + '_' + str(i)] = ParamVector(parents=[context['C' + str(j + 1)]], as_scalar=False, monitor=False)

n = len(y)
if n == 1:
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
    for i in range(0, int(NumLevs)):
        context['tau_u' + str(i)] = ParamScalar()
        context['sigma_u' + str(i)] = ParamScalar(modelled = False)
else:
    omega_e = ParamMatrix(modelled = False, customstep=True)
    omega_e.size = n
    d_e = ParamMatrix(customstep=True)
    d_e.size = n
    priors = Text('Priors (NB Uniform not supported by WinBUGS / OpenBUGS): ', ['Uniform', 'Wishart'])
    if priors == 'Wishart':
        R = List('R matrix: ')
        v = Integer('Degrees of Freedom:')
    for i in range(0, int(NumLevs)):
        context['omega_u' + str(i+1)] = ParamMatrix(modelled = False, customstep=True)
        context['omega_u' + str(i+1)].size = n
        context['d_u' + str(i+1)] = ParamMatrix(customstep=True)
        context['d_u' + str(i+1)].size = n
        context['priors' + str(i)] = Text('Priors at ' + context['C' + str(i + 1)] + ' classification (NB Uniform not supported by WinBUGS / OpenBUGS): ', ['Uniform', 'Wishart'])
        if context['priors' + str(i)] == 'Wishart':
            context['R' + str(i)] = List('R matrix: ')
            context['v' + str(i)] = Integer('Degrees of Freedom:')


beta = ParamVector()
beta.ncols = lenbeta

deviance = ParamScalar(modelled = False)
'''

    model = '''
<% lenbeta = 0 %> \\
<% numlevs = int(NumLevs) %> \\
<% n = len(y) %> \\
model {

        for (i in 1:length(${y[0]})) {
    % if n == 1:
            miss0[i]  ~ dnorm(mu[i], tau)
            mu0[i] <- ${mmult2(context['x1'], 'beta', 'i', lenbeta)} \\
% for i in range(0, numlevs):
+ u0_${i}[${context['C' + str(i + 1)]}[i]]
% endfor
<% lenbeta += len(context['x1']) %> \\
    % else:
            dummy[i] ~ dnormal${n}a( \\
        % for i in range(0,n):
miss${y[i]}[i], \\
        % endfor
        % for i in range(0,n):
mu${i}[i], \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_e[${i}], \\
        % endfor
d_e[${(((n+1)*n)/2) - 1}])
        % for i in range(0, len(y)):
            mu${i}[i] <- ${mmult2(context['x' + str(i+1)], 'beta', 'i', lenbeta)}\\
% for j in range(0, numlevs):
 + u${j}_${i}[${context['C' + str(j + 1)]}[i]]\\
% endfor
<% lenbeta += len(context['x' + str(i+1)]) %>
        % endfor
        % for i in range(0,n):
        miss${y[i]}[i] ~ dflat()
        % endfor
    % endif
        }

% for j in range(0, numlevs):
    for (i${j + 1} in 1:length(u${j}_0)) {
        %if len(y) == 1 :
           u${j}_0[i] ~ dnorm(0, tau_u)
    % else:
        dummy_u${j}[i${j + 1}] ~ dnormal${n}a( \\
        % for i in range(0,n):
u${j}_${i}[i${j + 1}], \\
        % endfor
        % for i in range(0,n):
0, \\
        % endfor
        % for i in range(0,(((n+1)*n)/2) - 1):
d_u${j+1}[${i}], \\
        % endfor
d_u${j+1}[${(((n+1)*n)/2) - 1}])
        dum_u${j}[i${j + 1}] ~ ddummy(dummy_u${j}[i${j + 1}])
        #above line to fix the DIC
        % for i in range(0,n):
        u${j}_${i}[i${j + 1}] ~ dflat()
        % endfor

    % endif
    }
% endfor
    # Priors

    % if n == 1:
    tau ~ dgamma(0.001, 0.001)
    sigma <- 1 / sqrt(tau)
    % for i in range(0, int(NumLevs)):
        tau_u${i} ~ dgamma(0.001, 0.001)
        sigma_u${i} <- 1 / sqrt(tau_u${i})
    % endfor
    % endif

    % for i in range(0, lenbeta):
    beta_${i} ~ dflat()
    % endfor
}
'''

    latex = r'''
<% lenbeta = 0 %>
<% numlevs = int(NumLevs) %>
\begin{aligned}
\left(
\begin{array}{l}
% for i in range(0, len(y)):
\mbox{${y[i]}}_i
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
\mu_{${i}i}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_e \right] \\ 
% for i in range(0, len(y)): 
\mu_{${i}i} & = ${mmulttex2(context['x' + str(i+1)], r'\beta', 'i', lenbeta)}
% for lev in range(0, numlevs):
 + u^{(${lev + 1})}_{${i},${context['C' + str(lev + 1)]}(i)}
% endfor
\\ 
<% lenbeta += len(context['x' + str(i+1)]) %>
% endfor
% for lev in range(0, numlevs):
\left(
\begin{array}{l}
% for i in range(0, len(y)):
u^{(${lev + 1})}_{${i},${context['C' + str(lev + 1)]}(i)}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
0
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega^{(${lev + 1})}_{u} \right] \\ 
% endfor
%if str(Engine) in ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS' , 'MLwiN_MCMC', 'R_MCMCglmm', 'R_nimble' ]:
% if priors == 'Wishart': 
\Omega_e & \sim \mbox{inverse Wishart}_{${len(y)}}[${v}*R_e,${v}],R_e=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${R.name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if priors == 'Uniform':
\Omega_e & \propto 1
% endif
\\ 
% for lev in range(0, numlevs):
% if context['priors' + str(lev)] == 'Wishart':
\Omega^{(${lev + 1})}_u & \sim \mbox{inverse Wishart}_{${len(y)}}[${context['v' + str(lev)]}*R_{u${lev + 1}},${context['v' + str(lev)]}],R_{u${lev + 1}}=
\left[\begin{array}{
% for i in range(0, len(y)):
l
% endfor
}
<% lenR = 0%>
% for i in range(0, len(y)):
% for j in range(0, len(y)):
% if j <= i:
${context['R' + str(lev)].name[lenR]}
<% lenR += 1 %>
% endif
% if j != len(y) - 1:
&
% else:
% if i != len(y) - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if context['priors' + str(lev)] == 'Uniform':
\Omega^{(${lev + 1})}_u & \propto 1
% endif
\\ 
% endfor
% for i in range(0, lenbeta):
\beta_{${i}} & \propto 1 \\ 
% endfor
%endif
\end{aligned}
'''

    preccode = '''
<%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += 'double(' + name + '[' + index + ']) * ' + var + '_' + str(count)
            count += 1
        return out
    def index(n, t):
        ind = ''
        for i in range(0, n):
            ind += '[i' + str(i + 1) + ']'
        if t != '':
            ind += '[' + t + ']'
        return ind
%>
    {
<% n = len(y) %>\\
<% numlevs = int(NumLevs) %>\\
        // Note currently using a uniform prior for variance matrix
        SymMatrix sbe(${n});
        int vwe = length(${y[0]});
<% numbeta = 0 %>\\
        for (int i = 0; i < length(${y[0]}); i++) {
% for k in range(0, n):
            double e${k} = double(miss${y[k]}[i]) - (\\
% for j in range(0, len(context['x' + str(k+1)])):
% if j != 0:
 + \\
% endif
double(${context['x' + str(k+1)][j]}[i]) * beta[${numbeta}] \\
<% numbeta += 1 %>\\
% endfor
)\\
% for i in range(0, numlevs):
 - double(u${i}_${k}[int(${context['C' + str(i + 1)]}[i])]) \\
% endfor
;

% endfor
% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += e${i} * e${j};
% endfor
% endfor
        }
% if priors == 'Uniform':
            if (runstate == 0) {
% for i in range(0, n):
                sbe(${i}, ${i}) += 0.0001;
% endfor
            }
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(R.name[count])
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
            sbe(${i}, ${j}) += ${str(Rmat[i, j] * float(v))};
% endfor
% endfor
% endif
            matrix_sym_invinplace(sbe);

% if priors == 'Uniform':
            vwe -= ${n + 1};
% endif
% if priors == 'Wishart':
            vwe += ${v};
% endif

            bool fail = false;
            mat_d_e = dwishart(vwe, sbe, fail);
            mat_omega_e = matrix_sym_inverse(mat_d_e, fail);

% for k in range(0, numlevs):
        std::vector<double*> tmp_u${k};
% for i in range(0, n):
        tmp_u${k}.push_back(u${k}_${i});
% endfor
        RectMatrix mat_u${k}(tmp_u${k}, length(u${k}_0));

        SymMatrix sbu${k} = mat_u${k}.T() * mat_u${k};

% if context['priors' + str(k)] == 'Uniform':
        if (runstate == 0) {
% for i in range(0, n):
            sbu${k}(${i}, ${i}) += 0.0001;
% endfor
        }
% endif
% if context['priors' + str(k)] == 'Wishart':
<%
import numpy
Rmat = numpy.empty([n, n])
count = 0
for i in range(0, n):
    for j in range(0, i + 1):
        Rmat[i, j] = float(context['R' + str(k)].name[count])
        Rmat[j, i] = Rmat[i, j]
        count += 1
%>

<% count = 0 %>
% for i in range(0, n):
% for j in range(i, n):
        sbu${k}(${i}, ${j}) += ${str(Rmat[i, j] * float(context['v' + str(k)]))};
% endfor
% endfor
% endif
        matrix_sym_invinplace(sbu${k});
% if context['priors' + str(k)] == 'Uniform':
        int vwu${k} = length(u${k}_0) - ${n + 1};
% endif
% if context['priors' + str(k)] == 'Wishart':
        int vwu${k} = length(u${k}_0) + ${context['v' + str(k)]};
% endif

        mat_d_u${k+1} = dwishart(vwu${k}, sbu${k}, fail);
        mat_omega_u${k+1} = matrix_sym_inverse(mat_d_u${k+1}, fail);
% endfor
    }
'''

    bugsmodel ='''
<% M = len(y) %>\\
<% lenbeta = 0 %>\\
        model {
            for (i in 1:N) {
% if M == 1:
                Y[i] ~ dnorm(mu0[i], tau)
                mu0[i] <- ${mmult2(context['x1'],'beta','i',lenbeta)}
<% lenbeta += len(context['x1']) %>\\
            }
% else:
                Y[i, 1:M] ~ dmnorm(mu[i, 1:M],d_e[1:M, 1:M])
% for i in range(0,len(y)):
                mu[i, ${i+1}] <- ${mmult2(context['x'+ str(i+1)],'beta','i',lenbeta)} \\
% for lev in range(0,int(NumLevs)):
 + u${lev+1}[${context['C'+str(lev +1)]}[i], ${i+1}]\\
% endfor

<% lenbeta += len(context['x' + str(i+1)]) %>\\
% endfor
            }
%endif
% for lev in range(0,int(NumLevs)):
            for (i in 1:n${lev+1}) {
               u${lev+1}[i, 1:M] ~ dmnorm(zero[1:M], d_u${lev+1}[1:M, 1:M])
            }
% endfor

            #Priors
            # for fixed effects
% for i in range(0, lenbeta):
            beta_${i} ~ dflat()
% endfor
            for (i in 1:M)
            {
               zero[i] <- 0
            }
            #for random terms
% if M==1:
            tau ~ dgamma(0.001, 0.001)
% else:
            d_e[1:M, 1:M] ~ dwish(R[,], ${v})
            omega_e[1:M, 1:M] <- inverse(d_e[1:M, 1:M])
% for lev in range(0,int(NumLevs)):
            d_u${lev+1}[1:M, 1:M] ~ dwish(R${lev+1}[,],${context['v'+str(lev)]})
            omega_u${lev+1}[1:M, 1:M] <- inverse(d_u${lev+1}[1:M, 1:M])
% endfor
% endif
        }
'''

    mlnscript = '''
<% N=datafile.nobs %>
<% M=len(context['y']) %>
% if M==1 :
ADDT "_levres"
SETV 1 "_levres"
FPAR 0 "_levres"
% for i in range(0, len(y)):
% for j in range(len(context['x'+str(i+1)])):
ADDT "${context['x'+str(i+1)][j]}"
% endfor
% endfor
% else:
% for i in range(len(y)):
MVAR 1 "${y[i]}"
% endfor
% for i in range(NumLevs):
IDEN ${i + 3} "${context['C' + str(i + 1)]}"
% endfor
IDEN 2 "_id"
IDEN 1 "resp_indicator"

<%
import numpy
T=numpy.eye(len(y), dtype='int')
%>
% for j in range(0, len(T)):
RPAT ${' '.join([str(k) for k in T[j]])}
% for i in range(0,len(context['x'+str(j+1)])):
ADDT "${context['x'+str(j+1)][i]}"
% endfor
ADDT "_levres"
RPAT
% endfor

% for i in range(len(y)):
% for j in range(1, NumLevs + 2):
SETV ${j + 1} "_levres.${i + 1}"
% endfor
FPAR 0 "_levres.${i + 1}"
% endfor

<%
import numpy
lenbeta = 0
n = len(y)
for i in range(0, len(y)):
    lenbeta += len(context['x'+str(i+1)])

join = ' '.join([str(i) for i in numpy.zeros(int(lenbeta),dtype='int')])
print 'NumLevs:' + str(int(NumLevs)) 
for i in range( int(NumLevs)-1,-1,-1):
    if context['priors' + str(i)] == 'Wishart':
        TR = ' '.join(context['R' + str(i)].name)
        join += ' 1 '+ TR + ' ' + str(context['v' + str(i)])
    else:
        join += ' 0 '

if priors == 'Wishart':
    TR = ' '.join(R.name)
    join += ' 1 ' + TR + ' ' + str(v)
else:
    join += ' 0 '
%>
JOIN ${join} 0 c1092
PRIOR c1092
% endif
XCLA 1
        '''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bf{Open}\text{ }\bf{Sample}\text{ } \bf{Worksheet} \text{ or }\bf{Open}\text{ } \bf{Worksheet} \text{ from the }\bf{File}\text{ menu.}\\ 
&\text{Select }\bf{datafile.dta}.\\ 
&\text{Select }\bf{Equations}\text{ from the }\bf{Model}\text{ menu.}\\ 
&\text{Click on the } \bf{Responses}\text{ button at the bottom of the window.}\\ 
% for i in range(len(y)):
&\text{Click on }\bf{${y[i]}}\text{ from the list that appears.}\\ 
% endfor
&\text{Click on the }\bf{Done}\text{ button on the }\bf{Responses} \text{ window.}\\ 
&\text{On the }\bf{Equations}\text{ window click on the response names to bring up the }\mathbf{Y}\text{ }\bf{variable}\text{ window}.\\ 
<%tind=["i","j","k","l","m"]%>
&\text{On the }\mathbf{Y}\text{ }\bf{variable}\text{ window change the number of levels to } \mathbf{${NumLevs+2}}-\mathbf{${''.join( tind[0:(NumLevs+2)])}}.\\ 
% for i in range(NumLevs):
&\text{In the }\bf{level} \text{ }\mathbf{${i+3}(${tind[i+2]}):}\text{ list, select } \bf{${context['C' + str(i + 1)]}}.\\  
% endfor
&\text{In the }\bf{level} \text{ }\mathbf{2(j):}\text{ list, select } \bf{\_id}.\\  
&\text{Click on the }\bf{Done}\text{ button.}\\ 
&\text{Click the }\bf{Add}\text{ }\bf{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bf{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bf{Add}\text{ }\bf{Separate}\text{ }\bf{coefficients}\text{ button.}\\ 
% for i in range(len(y)):
&\text{On the }\bf{Equations}\text{ window click on }\bf{\_levres.${y[i]}}.\\ 
&\text{Remove the tick in the }\bf{Fixed}\text{ }\bf{parameter}\text{ tick box.}\\ 
% for i in range(NumLevs):
&\text{Add the tick in the }\mathbf{${tind[i+2]}}(\bf{${context['C' + str(i + 1)]}\_long})\text{ tick box.}\\ 
% endfor
&\text{Add the tick in the }\mathbf{j}(\bf{\_id\_long})\text{ tick box.}\\ 
&\text{Click on the }\bf{Done}\text{ button.}\\ 
% endfor
<%
tt=list()
for i in range(0, len(y)):
    for j in range(len(context['x'+str(i+1)])):
        tt.append( context['x'+str(i+1)].name[j])

tt=list(set(tt))
%>
% for i in range(len(tt)):
&\text{Click the }\bf{Add}\text{ }\bf{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bf{${tt[i]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bf{Add}\text{ }\bf{Separate}\text{ }\bf{coefficients}\text{ button.}\\ 
% endfor
%for i in range(0, len(y)):
%for j in range(len(tt)):
%if tt[j] not in context['x'+str(i+1)].name:
&\text{On the }\bf{Equations}\text{ window click on }\bf{${tt[j]}.${y[i]}}.\\ 
&\text{Remove the tick in the }\bf{Fixed}\text{ }\bf{parameter}\text{ tick box.}\\ 
&\text{Click on the }\bf{Done}\text{ button.}\\ 
%endif
%endfor
%endfor
'''

    latexMLwiNextra = r'''
<%sigind=['v','f','g','h']%>
&\text{Select the }\bf{MCMC}\text{ tab from the }\bf{Estimation}\text{ }\bf{control}\text{ window.}\\ 
&\text{Select }\bf{MCMC}/\bf{Priors}\text{ from the }\bf{Model}\text{ menu.}\\ 
&\text{Select }\bf{Gamma}\text{ }\bf{prior}\text{ from the } \bf{MCMC}\text{ }\bf{Priors}\text{ window (this sets the prior for level 1 variance).}\\ 
%for i in range( int(NumLevs)-1,-1,-1):
%if context['priors' + str(i)] == 'Wishart':
&\text{Select }\bf{Informative}\text{ }\bf{prior} \text{ from the } \bf{MCMC}\text{ }\bf{Priors}\text{ window.}\\ 
&\text{Select }\Sigma_{${sigind[i]}} \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for j in range(len(context['R' + str(i)].name)):
%if j==(len(context['R' + str(i)].name)-1):
\text{and }\mathbf{${context['R' + str(i)].name[j]}}
%else:
\mathbf{${context['R' + str(i)].name[j]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${context['v' + str(i)]}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_{${sigind[i]}} \text{ again.}\\ 
%endif
%endfor
% if priors == 'Wishart':
&\text{Select }\Sigma_{u} \text{ from the top bar of the window.}\\ 
&\text{Input the estimates }
%for i in range(len(R.name)):
%if i==(len(R.name)-1):
\text{and }\mathbf{${R.name[i]}}
%else:
\mathbf{${R.name[i]}}\text{, }
%endif
%endfor
\text{ in the estimate boxes.}\\ 
&\text{Input } \mathbf{${v.name}} \text{ in the sample size box.}\\ 
&\text{Click on }\Sigma_u \text{ again.}\\ 
%endif
&\text{Click on the }\bf{Done}\text{ button.}\\

'''
    
    rscript = '''
<%
M=len(y)
resp='cbind('
nparams=0
for i in range(M):
    resp+=str(y[i])
    nparams+=len(context['x'+str(i+1)])
    if i!= M-1:
        resp+=','
    else:
        resp+=')'
TT=1
for i in range(M-1,0,-1):
    if len(context['x'+str(i+1)])!=len(context['x'+str(i)]):
        TT=0
    else:
        aa=context['x'+str(i+1)].name
        aa.sort()
        bb=context['x'+str(i)].name
        bb.sort()
        for j in range(len(aa)):
            if aa[j]!=bb[j]:
                TT=0
    
%>

% if Rpackage == 'MCMCglmm':
% if TT!=0:
formula <- ${resp} ~ trait:(${Rmmult(x1)}) - 1

##define the prior...use non-informative prior #fixed effect N(0,10^6), variance~IW(1,0.002)~IG(0.001000,0.001000)
#random effect Uniform: G1~IW(vw${i+1},diag(M)); Wishart: Rmat${i+1}
#Residual Uniform: G1~IW(vw,diag(M)*.0001); Wishart: Rmat
% for i in range(0, int(NumLevs)):
% if context['priors' + str(i)] == 'Uniform':
vw${i+1} = ${M + 1};
% endif
% if context['priors' + str(i)] == 'Wishart':
vw${i+1} = ${context['v' + str(i)]};
Rmat${i+1} = matrix(0,${M},${M})
Rvec${i+1}=rep(0,${len(context['R' + str(i)].name)})
% for j in range(0,len(context['R' + str(i)].name)):
Rvec${i+1}[${j+1}]=as.numeric(${float(context['R' + str(i)].name[j])})
% endfor
count = 0 
for (m in  0:${M-1}){
    for (n in 0:m ){
        Rmat${i+1}[m+1, n+1] = as.numeric(Rvec${i+1}[count+1])
        Rmat${i+1}[n+1, m+1] = Rmat${i+1}[m+1, n+1]
        count = count+ 1
    }
}
% endif
% endfor

% if priors.name == 'Uniform':
vw = ${M + 1};
% endif
% if priors.name == 'Wishart':
vw =  ${v.name};
Rmat = matrix(0,${M},${M})
Rvec=rep(0,${len(R.name)})
% for i in range(0,len(R.name)):
Rvec[${i+1}]=as.numeric(${float(R.name[i])})
% endfor
count = 0 
for (j in  0:${M-1}){
    for (k in 0:j ){
        Rmat[j+1, k+1] = as.numeric(Rvec[count+1])
        Rmat[k+1, j+1] = Rmat[j+1, k+1]
        count = count+ 1
    }
}
% endif
% if priors.name == 'Uniform':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(
% for i in range(0,int(NumLevs)):
G${i+1}=list(V=diag(${M}),nu=vw${i+1})
%if i!=int(NumLevs)-1:
,
%else:
)
% endif
% endfor
,R=list(V=diag(${M}),nu=vw))
% endif
% if priors.name == 'Wishart':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),G=list(
% for i in range(0,int(NumLevs)):
G${i+1}=list(V=Rmat${i+1},nu=vw${i+1})
%if i!=int(NumLevs)-1:
,
% else:
)
% endif
% endfor
,R=list(V=Rmat,nu=vw))
% endif


myModel<-MCMCglmm(formula,random=~\\
% for i in range(0,int(NumLevs)):
us(trait):${context['C'+str(i + 1)]}\\
% if i!=int(NumLevs)-1:
+
% endif
% endfor
,rcov=~us(trait):units,family=rep("gaussian",${M}),prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata,verbose=FALSE)



## summary and outputs from the model
## combined output fixed and variance into one matrix 
summary(myModel)
myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)
summary(myModelOut)

##temporary plots
<% ind=divmod(nparams,3) %>

% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor

% if (ind[1]>0):
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off()
% endif

## export chains
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")


% else:
<%print 'The model not implemented yet, choose another engine:'%>
% endif
% endif
'''

    preparedata = '''
mydata = data['datafile']

for i in range(0, len(y)):
    context['miss'+str(y[i])].ncols = -1*len(mydata.variables[y[i]]['data'])
'''

    bugsdata = '''
import numpy
        
# Y datamatrix
# X datamatrix with x1 for variable 1 with x1, x2 for variable 2
        
N=dsets['datafile'].nobs
M=len(y)

consts['N'] = N
consts['M'] = M
for i in range(0,int(NumLevs)):
    consts['n' + str(i+1)] = len(list(set(dsets['datafile'].variables[context['C' + str(i + 1)]]['data'])))
        
#start with R structure
Rmat=numpy.zeros((M,M))
k=0
        
for i in range(M):
    for j in range(i+1):
        Rmat[i,j]=float(R[k])
        Rmat[j,i]=float(R[k])
        k+=1

consts['R'] = Rmat


#Next do higher R structure
for lev in range(0,int(NumLevs)):
    Rmat = numpy.zeros((M,M))
    k = 0
        
    for i in range(M):
        for j in range(i+1):
            Rmat[i,j]=float(context['R' + str(lev)].name[k])
            Rmat[j,i]=float(context['R' + str(lev)].name[k])
            k+=1

    consts['R' + str(lev+1)] = Rmat


# Classification IDs
for lev in range(0,int(NumLevs)):
    idvar = str(context['C' + str(lev + 1)])
    uniq = numpy.unique(dsets['datafile'].variables[idvar]['data'].compressed())
    cid = numpy.zeros(len(dsets['datafile'].variables[idvar]['data']))
    map = zip(range(1, len(dsets['datafile'].variables[idvar]['data'])+1), uniq)
    d = {}
    for i in map:
        d[i[1]] = i[0]
    for i in range(0, len(dsets['datafile'].variables[idvar]['data'])):
        cid[i] = d[dsets['datafile'].variables[idvar]['data'][i]]
    consts[idvar] = cid

# x variables
for i in range(M):
    for j in range(len(context['x'+str(i+1)])):
        data[context['x'+str(i+1)][j]] = dsets['datafile'].variables[context['x'+str(i+1)][j]]['data']
                        
# Y structure
Ymat=numpy.ma.zeros((N,M))
for i in range(M):
    Ymat[:,i] = dsets['datafile'].variables[context['y'][i]]['data']

data['Y'] = Ymat
'''
